import os

from testtools import TestCase
from testtools.matchers import Contains
from . import makeprefs, _temp_home
from mock import MagicMock, Mock, patch
from StringIO import StringIO
from contextlib import nested


class ArchiveCommandTest(TestCase):
    def setUp(self):
        super(ArchiveCommandTest, self).setUp()
        self.prefs = makeprefs()
        self.home = os.path.join('t', 'data', 'home')

    def tearDown(self):
        super(ArchiveCommandTest, self).tearDown()

    def _makeit(self, *args, **kwargs):
        from lacli.main import LaArchiveCommand
        return LaArchiveCommand(*args, **kwargs)

    def _makeupload(self, *args, **kwargs):
        return Mock(upload=Mock(**kwargs))

    def test_command(self):
        assert self._makeit(Mock(), Mock(), self.prefs)

    @patch('sys.stdout', new_callable=StringIO)
    def test_do_archive_with_title(self, mock_stdout):
        cache = Mock(prepare=Mock(return_value='True'))
        cli = self._makeit(Mock(), cache, self.prefs)
        cli.onecmd('create {} baz'.format(self.home))
        cache.prepare.assert_called_with("baz", self.home)
        self.assertThat(mock_stdout.getvalue(),
                        Contains('archive prepared'))

    @patch('sys.stdout', new_callable=StringIO)
    def test_do_archive_exception(self, mock_stdout):
        cache = Mock(prepare=Mock(side_effect=Exception("foo")))
        cli = self._makeit(Mock(), cache, self.prefs)
        cli.onecmd('create {} baz'.format(self.home))
        self.assertThat(mock_stdout.getvalue(),
                        Contains('error: foo'))

    @patch('sys.stdout', new_callable=StringIO)
    def test_do_archive_unexist(self, mock_stdout):
        cli = self._makeit(Mock(), Mock(), self.prefs)
        cli.onecmd('create /tmp/doesnotexistisay foo')
        self.assertEqual('The specified folder does not exist.\n',
                         mock_stdout.getvalue())

    @patch('sys.stdout', new_callable=StringIO)
    def test_do_archive_list_none(self, out):
        cache = Mock(_for_adf=Mock(return_value=[]))
        cli = self._makeit(Mock(), cache, self.prefs)
        cli.onecmd('list')
        self.assertEqual('No available archives.\n', out.getvalue())

    @patch('sys.stdout', new_callable=StringIO)
    def test_do_archive_list_some(self, out):
        from lacli.adf import Archive, Meta
        meta = Meta(format='', size=None, cipher='')
        archive = Archive(title="foo", description='',
                          tags=[], meta=meta)
        cache = Mock(_for_adf=Mock(
            return_value={'foo': {'archive': archive}}))
        cli = self._makeit(Mock(), cache, self.prefs)
        cli.onecmd('list')
        self.assertThat(out.getvalue(),
                        Contains('Prepared archives:\n1) foo'))

    @patch('sys.stdout', new_callable=StringIO)
    def test_do_archive_list_verbose(self, out):
        from lacli.adf import Archive, Meta
        meta = Meta(format='', size=None, cipher='')
        archive = Archive(title="foo", description='',
                          tags=[], meta=meta)
        cache = Mock(_for_adf=Mock(
            return_value={'foo': {'archive': archive}}))
        prefs = self.prefs
        prefs['command']['verbose'] = True
        cli = self._makeit(Mock(), cache, self.prefs)
        cli.onecmd('list')
        self.assertThat(out.getvalue(),
                        Contains('!archive'))

    def test_do_archive_list_more(self):
        with patch('sys.stdout', new_callable=StringIO) as out:
            for size in [(25, '25B'), (1024, '1KiB'), (2000000, '1MiB')]:
                from lacli.adf import Archive, Meta
                meta = Meta(format='', size=size[0], cipher='')
                archive = Archive(title="foo", description='',
                                  tags=[], meta=meta)
                cache = Mock(_for_adf=Mock(
                    return_value={'foo': {'archive': archive}}))
                cli = self._makeit(Mock(), cache, self.prefs)
                cli.onecmd('list')
                self.assertThat(out.getvalue(),
                                Contains('Prepared archives:\n1) foo [//'
                                         + size[1]))

    @patch('sys.stdin', new_callable=StringIO)
    def test_loop_none(self, mock_stdin):
        cli = self._makeit(Mock(), Mock, self.prefs)
        cli.cmdloop()

    def test_do_status_error(self):
        from lacli.cache import Cache
        cli = self._makeit(Mock(), Cache(self.home), self.prefs)
        with patch('sys.stdout', new_callable=StringIO) as out:
            cli.onecmd('status foobar')
            self.assertThat(out.getvalue(),
                            Contains('error: invalid value'))
        apisession = Mock()
        apisession.upload_status.side_effect = Exception('foo')
        cli = self._makeit(apisession, Cache(self.home), self.prefs)
        with nested(
            patch('sys.stdout', new_callable=StringIO),
            patch('lacli.command.urlparse'),
                ) as (out, urlparse):
            uploads = cli.cache._for_adf('archives')
            urlparse.return_value = Mock(scheme='gopher', path=self.home)
            for seq, archive in enumerate(uploads.itervalues()):
                    if archive['archive'].title == 'My pending upload':
                        cli.onecmd('status {}'.format(seq+1))
            self.assertThat(out.getvalue(),
                            Contains('error:'))

    def test_do_status(self):
        from lacli.cache import Cache
        cli = self._makeit(Mock(), Cache(self.home), self.prefs)
        with patch('sys.stdout', new_callable=StringIO) as out:
            cli.onecmd('status')
            self.assertThat(out.getvalue(),
                            Contains('Usage:'))

    def test_do_put_error(self):
        from lacli.cache import Cache
        with _temp_home() as home:
            cli = self._makeit(Mock(), Cache(home), self.prefs)
            with patch('sys.stdout', new_callable=StringIO) as out:
                cli.onecmd('upload')
                self.assertThat(out.getvalue(),
                                Contains('No such archive'))
        cli = self._makeit(Mock(), Cache(self.home), self.prefs)
        with patch('sys.stdout', new_callable=StringIO) as out:
            cli.onecmd('upload foobar')
            self.assertThat(out.getvalue(),
                            Contains('error: invalid value'))
        with patch('sys.stdout', new_callable=StringIO) as out:
            archives = cli.cache._for_adf('archives')
            for seq, archive in enumerate(archives.itervalues()):
                if archive['archive'].title == 'My 2013 vacation':
                    cli.onecmd('upload {}'.format(seq+1))
            self.assertThat(out.getvalue(),
                            Contains('upload is already completed'))

    def test_do_put_not_found(self):
        from lacli.cache import Cache
        cli = self._makeit(Mock(), Cache(self.home), self.prefs)
        cli.cache.links = Mock(return_value={})
        with patch('sys.stdout', new_callable=StringIO) as out:
            for seq, archive in enumerate(cli.cache._for_adf('archives')):
                if archive.title == 'My 2013 vacation':
                    cli.onecmd('upload {}'.format(seq+1))
            self.assertThat(out.getvalue(), Contains('no local copy exists'))

    def test_do_put_not_local(self):
        from lacli.cache import Cache
        cli = self._makeit(Mock(), Cache(self.home), self.prefs)
        with patch('sys.stdout', new_callable=StringIO) as out:
            with patch('lacli.command.urlparse') as urlparse:
                urlparse.return_value = Mock(scheme='gopher', path=self.home)
                for seq, archive in enumerate(cli.cache._for_adf('archives')):
                    if archive.title == 'My 2013 vacation':
                        cli.onecmd('upload {}'.format(seq+1))
                self.assertThat(out.getvalue(), Contains('url not local'))
                urlparse.assert_called_with('file:///path/to/archive')

    def test_do_put_done(self):
        with patch('sys.stdout', new_callable=StringIO) as out:
            from lacli.cache import Cache
            upload = MagicMock()
            upload.__enter__.return_value = {
                'uri': 'http://foo.com',
                'id': '1',
                'tokens': Mock()
            }
            with patch.object(Cache, 'save_upload'):
                cli = self._makeit(Mock(upload=Mock(return_value=upload)),
                                   Cache(self.home),
                                   self.prefs,
                                   self._makeupload())
                with patch('lacli.command.urlparse') as urlparse:
                    urlparse.return_value = Mock(scheme='file', path=self.home)
                    archives = cli.cache._for_adf('archives')
                    for seq, archive in enumerate(archives):
                        if archive.title == 'My 2013 vacation':
                            cli.onecmd('upload {}'.format(seq+1))
            self.assertThat(out.getvalue(), Contains('done'))

    def test_do_put_exception(self):
        with patch('sys.stdout', new_callable=StringIO) as out:
            from lacli.cache import Cache
            upload = MagicMock()
            upload.__enter__.return_value = {
                'uri': 'http://foo.com',
                'id': '1',
                'tokens': Mock()
            }
            cli = self._makeit(Mock(upload=Mock(return_value=upload)),
                               Cache(self.home), self.prefs,
                               self._makeupload(side_effect=Exception))
            with patch('lacli.command.urlparse') as urlparse:
                urlparse.return_value = Mock(scheme='file', path=self.home)
                for seq, archive in enumerate(cli.cache._for_adf('archives')):
                    if archive.title == 'My 2013 vacation':
                        cli.onecmd('upload {}'.format(seq+1))
            self.assertThat(out.getvalue(),
                            Contains('error:'))

    def test_do_restore_none(self):
        with patch('sys.stdout', new_callable=StringIO) as out:
            cache = Mock(archives=Mock(return_value=[]))
            cli = self._makeit(Mock(), cache, self.prefs, Mock())
            cli.onecmd('extract')
            self.assertThat(out.getvalue(), Contains('No such archive'))
            cli.onecmd('restore 1')
            self.assertThat(out.getvalue(), Contains('No such archive'))
            cli.onecmd('restore foobar')
            self.assertThat(out.getvalue(), Contains('No such archive'))

    @patch('lacli.command.restore_archive')
    def test_do_restore(self, restore_archive):
        with patch('sys.stdout', new_callable=StringIO) as out:
            cache = Mock(archives=Mock(return_value=[Mock(title='foo')]),
                         certs=Mock(return_value={}),
                         links=Mock(return_value={}))
            cli = self._makeit(Mock(), cache, self.prefs, Mock())
            cli.onecmd('extract')
            self.assertThat(out.getvalue(),
                            Contains('no matching certificate found'))
        with patch('sys.stdout', new_callable=StringIO) as out:
            cache.certs.return_value = {'foo': 'cert'}
            cli.onecmd('extract')
            self.assertThat(out.getvalue(), Contains('no local copy exists'))
        with patch('sys.stdout', new_callable=StringIO) as out:
            cache.links.return_value = {'foo': Mock(local="file:///path")}
            with patch('lacli.command.urlparse') as urlparse:
                urlparse.return_value = Mock(scheme='gopher', path=self.home)
                cli.onecmd('extract')
                self.assertThat(out.getvalue(), Contains('url not local'))
        with patch('sys.stdout', new_callable=StringIO) as out:
            cache.links.return_value = {'foo': Mock(local="file:///path")}
            cli.onecmd('extract')
            self.assertThat(out.getvalue(), Contains('archive restored'))
        with patch('sys.stdout', new_callable=StringIO) as out:
            cache.links.return_value = {'foo': Mock(local="file:///path")}
            cli.onecmd('extract')
            self.assertThat(out.getvalue(), Contains('archive restored'))
            args, kwargs = restore_archive.call_args
            a, p, cert, o, c, cb = args
            self.assertEqual('cert', cert)
            self.assertEqual('/path', p)
            self.assertEqual('foo', o)
            cb("foo")
            self.assertThat(out.getvalue(), Contains("Extracting foo"))
        with patch('sys.stdout', new_callable=StringIO) as out:
            restore_archive.side_effect = Exception("foo")
            cache.links.return_value = {'foo': Mock(local="file:///path")}
            cli.onecmd('extract')
            self.assertThat(out.getvalue(), Contains('error: foo'))

    @patch('sys.stdin', new_callable=StringIO)
    def test_do_archive(self, mock_stdin):
        cli = self._makeit(Mock(), Mock, self.prefs)
        cli.onecmd('archive')



import logging
import multiprocessing
from logutils.queue import QueueListener
from boto import config as boto_config
from twisted.internet.defer import setDebugging as debugTwisted
from twisted.python import log as twisted_log
from contextlib import contextmanager


simplefmt = '%(name)-15s %(levelname)-8s %(processName)-10s %(message)s'
logging.basicConfig(format=simplefmt)


def setupLogging(level, logfile=None, queue=False):
    # this overrides any user debug setting in the boto configuration
    if not boto_config.has_section('Boto'):
        boto_config.add_section('Boto')

    import httplib
    httplib.HTTPConnection.debuglevel = level

    if level == 0:
        level = 'WARN'
        boto_config.set('Boto', 'debug', '0')
    elif level == 1:
        level = 'INFO'
        boto_config.set('Boto', 'debug', '0')
    elif level == 2:
        level = 'DEBUG'
        boto_config.set('Boto', 'debug', '1')
    else:
        level = 'DEBUG'
        boto_config.set('Boto', 'debug', '2')
        debugTwisted(True)

    logconf = {
        'version': 1,
        'formatters': {
            'simple': {
                'class': 'logging.Formatter',
                'format': simplefmt,
            }
        },
        'handlers': {
            'console': {
                'class': 'logging.StreamHandler',
                'level': level,
                'formatter': 'simple',
            }
        },
        'loggers': {
            'boto': {
                'handlers': ['console'],
                'propagate': True
            },
            'lacli': {
                'handlers': ['console'],
                'propagate': True
            },
            'multiprocessing': {
                'handlers': ['console'],
                'propagate': True
            },
            'requests.packages.urllib3': {
                'handlers': ['console'],
                'propagate': True
            },
            'twisted': {
                'handlers': ['console'],
                'propagate': True
            }
        },
        'root': { 'level': level }
    }

    logging.config.dictConfig(logconf)

    multiprocessing.get_logger()  # initialize mp logging

    return log_open(logfile)


@contextmanager
def log_open(log):
    logfile = None
    if log is not None:
        try:
            logfile = open(log, 'w+')
            handler = logging.StreamHandler(logfile)
            handler.setFormatter(logging.Formatter(simplefmt))
            logging.getLogger('').addHandler(handler)
        except Exception as e:
            getLogger().debug("couldn't open log", exc_info=True)

    observer = twisted_log.PythonLoggingObserver()
    observer.start()

    yield None

    observer.stop()

    if logfile is not None:
        logfile.close()

class queueHandler(object):
    def __enter__(self):
        q = multiprocessing.Queue()
        self.listener = QueueListener(q, self)
        self.listener.start()
        return q

    def __exit__(self, type, value, traceback):
        self.listener.stop()


class LogHandler(queueHandler):
    def __init__(self, logger='lacli'):
        self.logger = getLogger(logger)

    def handle(self, msg):
        self.logger.handle(msg)


def logToQueue(queue):
    logging.config.dictConfig({
        'version': 1,
        'disable_existing_loggers': True,
        'handlers': {
            'queue': {
                'class': 'logutils.queue.QueueHandler',
                'queue': queue,
            },
        },
        'loggers': {
            'boto': {
                'handlers': ['queue']
            },
            'lacli': {
                'level': 'DEBUG',
                'handlers': ['queue']
            },
        },
        'root': {
            'level': 'DEBUG',
        },
    })


def getLogger(logger='lacli'):
    return logging.getLogger(logger)



import os

from testtools import TestCase
from testtools.matchers import Contains
from . import makeprefs
from mock import Mock, patch
from StringIO import StringIO


class CommandTest(TestCase):
    def setUp(self):
        super(CommandTest, self).setUp()
        self.prefs = makeprefs()
        self.home = os.path.join('t', 'data', 'home')

    def tearDown(self):
        super(CommandTest, self).tearDown()

    def _makeit(self, *args, **kwargs):
        from lacli.main import LaCommand
        return LaCommand(*args, **kwargs)

    def test_command(self):
        assert self._makeit(Mock(), Mock(), self.prefs)

    @patch('sys.stdin', new_callable=StringIO)
    def test_loop_none(self, mock_stdin):
        cli = self._makeit(Mock(), Mock, self.prefs)
        cli.cmdloop()

    @patch('sys.stdout', new_callable=StringIO)
    def test_dispatch(self, stdout):
        cli = self._makeit(Mock(), Mock, self.prefs)
        cli.dispatch('foo', [])
        self.assertThat(stdout.getvalue(),
                        Contains('Unrecognized command: foo'))

    @patch('sys.stdout', new_callable=StringIO)
    def test_dispatch_foo(self, stdout):
        cli = self._makeit(Mock(), Mock, self.prefs)
        with patch.object(cli, 'foo', create=True) as foo:
            foo.__doc__ = "Usage: lacli foo"
            foo.makecmd.return_value = 'bar'
            cli.dispatch('foo', [])
            self.assertEqual('', stdout.getvalue())
            foo.onecmd.assert_called_with('bar')

    @patch('sys.stdout', new_callable=StringIO)
    def test_dispatch_login(self, stdout):
        cli = self._makeit(Mock(), Mock, self.prefs)

        cli.dispatch('login', [])
        self.assertThat(stdout.getvalue(),
                        Contains('authentication succesfull'))

    @patch('sys.stdout', new_callable=StringIO)
    def test_do_login(self, stdout):
        cli = self._makeit(Mock(), Mock, self.prefs)
        cli.onecmd('login')
        self.assertThat(stdout.getvalue(),
                        Contains('authentication succesfull'))

    @patch('sys.stdout', new_callable=StringIO)
    def test_do_login_with_creds(self, stdout):
        cli = self._makeit(Mock(), Mock, self.prefs)
        cli.onecmd('login username password')
        self.assertThat(stdout.getvalue(),
                        Contains('authentication succesfull'))

    @patch('sys.stdout', new_callable=StringIO)
    def test_do_login_with_bad_creds(self, stdout):
        api = Mock()
        api.set_session_factory.side_effect = Exception()
        cli = self._makeit(api, Mock, self.prefs)
        self.assertRaises(Exception, cli.onecmd, 'login username password')



try: 
    from blessings import Terminal
    WITH_BLESSINGS = True
except ImportError:
    WITH_BLESSINGS = False

import sys
import time
from lacli.adf import creation

def format_size(bytes):
    size = ""
    if bytes:
        kib = bytes // 1024
        mib = kib // 1024
        gib = mib // 1024
        if not kib:
            size = "{} B".format(bytes)
        elif not mib:
            size = "{} KB".format(kib)
        elif not gib:
            size = "{} MB".format(mib)
        else:
            size = "{} GB".format(gib)
    return size

class RichTextUI:
    def __init__(self):

        if WITH_BLESSINGS:
            t = Terminal()
            self.width = t.width
        else:
            self.width = 78
        if not self.width:
            self.width = 78

        h_titles = '#   ID         STATUS    SIZE   DATE         TITLE'
        h_pattern= '--- ---------- --------- ------ ------------ ---------------------------------'
        h_pattern= h_pattern.ljust(self.width, '-')
        self.archive_design = {
            'titles': h_titles,
            'pattern': h_pattern,
            'frmt': "{:0%sd} {:>%s} {:<%s} {:>%s} {:<%s} {:<%s}" % tuple( len(p) for p in h_pattern.split(' ')) 
        }

        h_titles = 'ID         SIZE   DATE       TITLE'
        h_pattern= '---------- ------ ---------- ---------------------------------'
        h_pattern= h_pattern.ljust(self.width, '-')
        self.cert_design = {
            'titles': h_titles,
            'pattern': h_pattern,
            'frmt': "{:>%s} {:>%s} {:<%s} {:<%s}" % tuple( len(p) for p in h_pattern.split(' ')) 
        }

        h_titles = '#  ID      SIZE   FREE CREATED      EXPIRES      TITLE'
        h_pattern= '-- ----- ------ ------ ------------ ------------ ------------------------------'
        h_pattern= h_pattern.ljust(self.width, '-')
        self.capsule_design = {
            'titles': h_titles,
            'pattern': h_pattern,
            'frmt': "{:0%sd} {:>%s} {:>%s} {:>%s} {:<%s} {:<%s} {:<%s}" % tuple( len(p) for p in h_pattern.split(' ')) 
        }

    def print_archives_header(self):
        print self.archive_design['titles']
        print self.archive_design['pattern']

    def print_archives_line(self, archive):
        h_parts = self.archive_design['pattern'].split()
        if len(archive['title'])>=len(h_parts[-1]):
            archive_title = archive['title'][0:len(h_parts[-1])-2]+'..'
        else:
            archive_title = archive['title']
        archive_title = archive_title.replace('\n',' ')
        print self.archive_design['frmt'].format(
            archive['num'],
            archive['cert'],
            archive['status'],
            archive['size'],
            archive['created'].strftime('%Y-%m-%d'),
            archive_title.encode('utf-8')) 
    
    def print_capsules_header(self):
        print self.capsule_design['titles']
        print self.capsule_design['pattern']
    
    def print_capsules_line(self, capsule):
        h_parts = self.capsule_design['pattern'].split()
        if len(capsule['title'])>=len(h_parts[-1]):
            capsule_title = capsule['title'][0:len(h_parts[-1])-2]+'..'
        else:
            capsule_title = capsule['title']
        print self.capsule_design['frmt'].format(
            capsule['num'],
            capsule['id'],
            format_size(capsule['size']),
            format_size(capsule['remaining']),
            capsule['created'].strftime('%Y-%m-%d'),
            capsule['expires'].strftime('%Y-%m-%d'),
            capsule_title.encode('utf-8')) 
    
    def print_certificates_header(self):
        print self.cert_design['titles']
        print self.cert_design['pattern']

    def print_certificates_line(self, certificate):
        h_parts = self.cert_design['pattern'].split()
        if len(certificate['title'])>=len(h_parts[-1]):
            certificate_title = certificate['title'][0:len(h_parts[-1])-2]+'..'
        else:
            certificate_title = certificate['title']
        certificate_title = certificate_title.replace('\n', ' ')
        print self.cert_design['frmt'].format(
            certificate['aid'],
            certificate['size'],
            certificate['created'].strftime('%Y-%m-%d'),
            certificate_title.encode('utf-8')) 
    
    def print_certificates_list(self, certs, debug):
        if len(certs):
            self.print_certificates_header()
            for cert in sorted(certs.itervalues(), key=creation):
                aid = cert['signature'].aid
                title = cert['archive'].title
                size = format_size(cert['archive'].meta.size)
                self.print_certificates_line( certificate={
                        'aid': aid,
                        'size':size,
                        'title': title,
                        'created':cert['archive'].meta.created
                    })
                if debug > 2:
                    for doc in cert.itervalues():
                        pyaml.dump(doc, sys.stdout)
                    print
            return True
        else:
            return False

    def countdown(self, num):
        print num,
        for n in reversed(range(num-1)):
            sys.stdout.flush()
            time.sleep(1)
            sys.stdout.write(", {}".format(n+1))
            yield ""



import os
import shutil

from behave_cli import environment as clienv


def before_all(context):
    clienv.before_all(context)


def after_all(context):
    clienv.after_all(context)


def before_feature(context, feature):
    clienv.before_feature(context, feature)


def after_feature(context, feature):
    clienv.after_feature(context, feature)


def before_scenario(context, scenario):
    clienv.before_scenario(context, scenario)
    context.archive = None
    context.cert = None


def after_scenario(context, scenario):
    clienv.after_scenario(context, scenario)
    if context.archive is not None:
        context.archive.close()
    if context.cert is not None:
        context.cert.close()
    d = os.path.join(context.environ['HOME'], ".longaccess")
    if os.path.isdir(d):
        shutil.rmtree(d)



"""encrypt a file using simple AES-CTR-256

Usage: laenc <file>
"""

import sys
from docopt import docopt
from tempfile import NamedTemporaryFile
from lacli.crypt import CryptIO
from lacli.cipher import new_key
from lacli.cipher.aes import CipherAES
from shutil import copyfileobj



def main(args=sys.argv[1:]):
    options = docopt(__doc__)
    with open(options['<file>']) as f:
        with NamedTemporaryFile(delete=False, mode='w') as out:
            key = new_key(256)
            with CryptIO(out, CipherAES(key), mode='w') as cf :
                copyfileobj(f, cf)
                print "saved in", out.name, "key is", key.encode('hex')

if __name__ == "__main__":
    main()
    
    



from __future__ import division
import os
import math
from lacli.date import parse_timestamp, remaining_time
from lacli.progress import make_progress, save_progress
from itertools import imap, repeat, izip
from lacli.log import getLogger
from lacli.exceptions import UploadEmptyError, WorkerFailureError, PauseEvent
from lacli.control import readControl
from boto import connect_s3
from boto.utils import compute_md5
from boto.s3.key import Key
from filechunkio import FileChunkIO
from sys import maxint
from tempfile import mkdtemp, mkstemp
from multiprocessing import TimeoutError, active_children


class MPConnection(object):
    def __init__(self, token, grace=120):
        self.accesskey = token['token_access_key']
        self.secret = token['token_secret_key']
        self.sectoken = token['token_session']
        self.expiration = token['token_expiration']
        self.uid = token['token_uid']
        self.bucket = token['bucket']
        self.grace = grace
        self.conn = None
        self.expiration = None
        if token.get('token_expiration', None):
            try:
                self.expiration = parse_timestamp(
                    token['token_expiration'])
            except ValueError:
                getLogger().debug("invalid token expiration: %s",
                                  token['token_expiration'])
        if self.uid is None:
            self.uid = self.getconnection().get_canonical_user_id()

    def getconnection(self):
        if self.conn is None:
            self.conn = connect_s3(
                aws_access_key_id=self.accesskey,
                aws_secret_access_key=self.secret,
                security_token=self.sectoken)
        return self.conn

    def getbucket(self):
        return self.getconnection().get_bucket(self.bucket)

    def newkey(self, key):
        return Key(self.getconnection().get_bucket(self.bucket), key)

    def timeout(self):
        """ return total number of seconds till
            this connection must be renewed. """
        if not self.expiration:
            return None
        remaining = remaining_time(self.expiration)
        return remaining.total_seconds() - self.grace

    def __getstate__(self):
        state = self.__dict__
        state['conn'] = None
        return state


class MPFile(object):
    minchunk = 5242880
    maxchunk = 104857600

    def __init__(self, path, skip=0):
        self.path = path
        self.isfile = os.path.isfile(path)
        size = maxint
        if self.isfile:
            size = os.path.getsize(path)
        assert size >= skip
        self.skip = skip
        self.size = size-skip
        self.chunk = min(max(int(self.size/100), self.minchunk), self.maxchunk)
        self.chunks = int(math.ceil(self.size/self.chunk))
        if self.chunks == 0:
            self.chunks = 1

    def _savedchunks(self, tempdir):
        # split file and save parts to disk
        f = open(self.path, "rb")

        def _save(seq):
            b = f.read(self.chunk)
            if b:
                prefix = "part-{:>4}".format(seq)
                fh, fn = mkstemp(dir=tempdir, prefix=prefix)
                os.write(fh, b)
                os.close(fh)
                return fn
            f.close()

        return imap(_save, xrange(self.chunks))

    def chunkstart(self, num):
        return self.skip + num * self.chunk

    def chunksize(self, num):
        start = num * self.chunk
        if (start + self.chunk > self.size):
            # chunk end is EOF
            return self.size - start
        else:
            return self.chunk

    def chunkfile(self, seq, fname):
        if self.isfile:
            return FilePart(self, seq)
        else:
            return SavedPart(self, seq, fname)


class FileHash(object):
    def __get__(self, instance, owner):
        try:
            return (instance.md5, instance.b64md5)
        except AttributeError:
            instance.seek(0)
            (instance.md5, instance.b64md5, size) = compute_md5(
                instance, instance.bytes)
            instance.seek(0)
            return (instance.md5, instance.b64md5)


class SavedPart(file):
    def __init__(self, source, num, *args, **kwargs):
        super(SavedPart, self).__init__(*args, **kwargs)
        self.bytes = source.chunksize(num)

    hash = FileHash()


class FilePart(FileChunkIO):

    def __init__(self, source, num, *args, **kwargs):
        super(FilePart, self).__init__(
            source.path, 'r',
            offset=source.chunkstart(num),
            bytes=source.chunksize(num), *args, **kwargs)

    hash = FileHash()


class MPUpload(object):

    def __init__(self, connection, source, key, step=None, retries=4):
        self.connection = connection
        self.source = source
        self.retries = retries
        self.key = key
        self.upload_id = None
        self.complete = None
        self.tempdir = None
        self.bucket = None
        self.step = step

    def __str__(self):
        return "<MPUpload key={} id={} source={}>".format(
            self.key, self.upload_id, self.source)

    def iterargs(self, chunks):
        partinfo = repeat(None)
        if not self.source.isfile:
            partinfo = self.source._savedchunks(self.tempdir)
        for seq, info in izip(xrange(chunks), partinfo):
            yield {'uploader': self, 'seq': seq, 'fname': info}

    def _getupload(self):
        if self.bucket is None:
            self.bucket = self.connection.getbucket()

        if self.source.chunks > 1 and self.source.isfile:
            if self.upload_id is None:
                return self.bucket.initiate_multipart_upload(self.key)

            for upload in self.bucket.get_all_multipart_uploads():
                if self.upload_id == upload.id:
                    return upload
        else:
            return self.connection.newkey(self.key)

    def __enter__(self):
        self.upload = self._getupload()
        if not hasattr(self.upload, 'set_contents_from_file'):
            self.upload_id = self.upload.id
            getLogger().debug("multipart upload with id: %s", self.upload_id)
        if not self.source.isfile:
            self.tempdir = mkdtemp()
        return self

    def __exit__(self, type, value, traceback):
        if value is not None:
            getLogger().debug("cancelling upload..", exc_info=True)
            if hasattr(self.upload, 'cancel_upload'):
                self.upload.cancel_upload()
        if self.tempdir is not None:
            os.rmdir(self.tempdir)
        return type is None

    def submit_job(self, pool):
        getLogger().debug("total of %d upload jobs for workers..",
                          self.source.chunks)
        chunks = self.source.chunks
        step = self.step
        if step is None:
            step = len(active_children()) or 5
        if chunks > step:
            chunks = step    
        return pool.imap(upload_part, self.iterargs(chunks))

    def complete_multipart(self, etags):
        xml = '<CompleteMultipartUpload>\n'
        for seq, etag in enumerate(etags):
            xml += '  <Part>\n'
            xml += '    <PartNumber>{}</PartNumber>\n'.format(seq+1)
            xml += '    <ETag>{}</ETag>\n'.format(etag)
            xml += '  </Part>\n'
        xml += '</CompleteMultipartUpload>'
        return self.bucket.complete_multipart_upload(
            self.upload.key_name, self.upload.id, xml)

    def get_result(self, rs):
        etags = []
        key = name = None
        newsource = None
        if rs is not None:
            try:
                while True:
                    key = rs.next(self.connection.timeout())
                    getLogger().debug("got key {} with etag: {}".format(key.name, key.etag))
                    etags.append(key.etag)
            except StopIteration:
                pass
            except WorkerFailureError:
                getLogger().debug("error getting result.", exc_info=True)

        if not etags:
            raise UploadEmptyError()

        if hasattr(self.upload, 'complete_upload'):
            key = self.complete_multipart(etags)
            name = key.key_name
        else:
            name = key.name

        uploaded = len(etags)
        total = self.source.chunks
        getLogger().debug("Uploaded {} out of {} chunks".format(uploaded, total))
        size = self.source.size
        if uploaded < total:
            for seq in xrange(uploaded, total):
                make_progress({'part': seq, 'tx': 0})
            skip = self.source.chunkstart(uploaded)
            newsource = MPFile(self.source.path, skip)
            size = size - newsource.size
        getLogger().debug("saving progress for {}".format(key))
        save_progress(name, size)
        return (key.etag, newsource)

    def do_part(self, seq, **kwargs):
        """ transfer a part. runs in a separate process. """

        key = None
        for attempt in range(self.retries):
            getLogger().debug("attempt %d/%d to transfer part %d",
                              attempt, self.retries, seq)

            with self.source.chunkfile(seq, **kwargs) as part:
                try:
                    def cb(tx, total):
                        readControl()
                        make_progress({'part': seq,
                                       'tx': tx,
                                       'total': total})

                    if hasattr(self.upload, 'upload_part_from_file'):
                        key = self.upload.upload_part_from_file(
                            fp=part,
                            part_num=seq+1,
                            cb=cb,
                            num_cb=100,
                            size=self.source.chunksize(seq),
                            # although not necessary, boto does it,
                            # good to know how:
                            md5=part.hash
                        )
                        getLogger().debug("uploaded multi part key %s/%d",
                                          key, seq+1)
                    else:
                        self.upload.set_contents_from_file(
                            fp=part,
                            cb=cb,
                            num_cb=100,
                            md5=part.hash
                        )
                        key = self.upload
                        getLogger().debug("uploaded single part key %s", key)
                except PauseEvent:
                    raise    
                except Exception as exc:
                    getLogger().debug("exception while uploading part %d",
                                      seq, exc_info=True)

                    if attempt == self.retries-1:
                        raise exc
                else:
                    getLogger().debug("succesfully uploaded %s part %d: %s",
                                      self.source, seq, key)
                    return key


def upload_part(kwargs):
    try:
        seq = kwargs.pop('seq')
        uploader = kwargs.pop('uploader')
        return uploader.do_part(seq, **kwargs)
    except PauseEvent:
        raise
    except Exception as e:
        raise WorkerFailureError(e)
# vim: et:sw=4:ts=4



#!/usr/bin/env python
# -*- coding: utf-8 -*-
from __future__ import print_function
import sys
import os
from PySide.QtCore import QUrl, QObject, Slot, Qt
from PySide.QtGui import QApplication, QFileDialog, QVBoxLayout, QWidget
from PySide.QtDeclarative import QDeclarativeView
import lacli.views.qrc_decrypt

class MainWindow(QDeclarativeView):
   
    def __init__(self, parent=None):
        super(MainWindow, self).__init__(parent)
        self.setWindowTitle("Certificate entry")
        # QML resizes to main window
        self.setResizeMode(QDeclarativeView.SizeViewToRootObject)
	#Decrypt Create an URL to the QML file
	url = QUrl('qrc:/ui/KeyInput.qml')
	# Set the QML file and show
	self.setSource(url)

# Create Qt application and the QDeclarative view
app = QApplication(sys.argv)
window = QWidget()
view = MainWindow()
layout = QVBoxLayout()
layout.addWidget(view)
window.setLayout(layout)
window.setWindowFlags(Qt.FramelessWindowHint)

class OutputSelector(QObject):
    def __init__(self):
        super(OutputSelector,self).__init__()

    @Slot(result=str)
    def getDirectory(self):
        return QFileDialog.getExistingDirectory(
            view, 'Select destination directory',
            view.rootObject().property('folder'))

    @Slot(str, result=bool)
    def dirExists(self, path):
        return os.path.isdir(path)

view.rootContext().setContextProperty("destsel", OutputSelector())



from lacli.decorators import command, block
from lacli.command import LaBaseCommand
from lacli.log import getLogger
from lacli.exceptions import ApiAuthException
from twisted.internet import defer
from re import match, IGNORECASE
from getpass import getpass


class LaLoginCommand(LaBaseCommand):
    """Login to Longaccess

    Usage: lacli login [<username> <password>]
    """
    prompt = 'lacli:login> '
    email = None

    def makecmd(self, options):
        cmd = ["login"]
        if options['<username>']:
            cmd.append(options['<username>'])
            if options['<password>']:
                cmd.append(options['<password>'])

        return " ".join(cmd)

    @property
    def username(self):
        return self.prefs['api']['user']

    @username.setter
    def username(self, newuser):
        self.prefs['api']['user'] = newuser

    @property
    def password(self):
        return self.prefs['api']['pass']

    @password.setter
    def password(self, newpassword):
        self.prefs['api']['pass'] = newpassword

    @command(username=str, password=str)
    def do_login(self, username=None, password=None):
        """
        Usage: login [<username>] [<password>]
        """

        save = (self.username, self.password)

        if not username and not self.batch:
            username = self.input("Username/email: ")

        if not password and not self.batch:
            password = getpass("Password: ")

        try:
            self.login_batch(username, password)
            print "authentication succesfull as", self.email
            if not self.batch:
                if self.username != save[0] or self.password != save[1]:
                    if match('y(es)?$',
                             self.input("Save credentials? "), IGNORECASE):
                        self.registry.save_session(self.username, self.password)
        except Exception as e:
            getLogger().debug("auth failure", exc_info=True)
            print "authentication failed"


    def login_batch(self, username, password):
        block(self.login_async)(username, password)

    @defer.inlineCallbacks
    def login_async(self, username, password):
        self.username = username
        self.password = password
        session = self.registry.new_session()
        try:
            account = yield session.async_account
            self.email = account['email']
            self.session = session
            getLogger().debug("logged in {}".format(self.email))
        except Exception as e:
            self.username = self.password = None
            getLogger().debug("auth failure", exc_info=True)
            raise ApiAuthException(username=username, exc=e)

    @command()
    def do_logout(self):
        """
        Usage: logout
        """
        self.logout_batch()

    def logout_batch(self):
        self.username = None
        self.password = None
        self.email = None
        self.registry.session = None



#
# Autogenerated by Thrift Compiler (0.9.0)
#
# DO NOT EDIT UNLESS YOU ARE SURE THAT YOU KNOW WHAT YOU ARE DOING
#
#  options string: py:twisted,new_style
#

from thrift.Thrift import TType, TMessageType, TException, TApplicationException

from thrift.transport import TTransport
from thrift.protocol import TBinaryProtocol, TProtocol
try:
  from thrift.protocol import fastbinary
except:
  fastbinary = None


class ErrorType(object):
  NoError = 0
  Server = 1
  Network = 2
  Authentication = 3
  Validation = 4
  Other = 5
  NotImplemented = 6
  FileNotFound = 7

  _VALUES_TO_NAMES = {
    0: "NoError",
    1: "Server",
    2: "Network",
    3: "Authentication",
    4: "Validation",
    5: "Other",
    6: "NotImplemented",
    7: "FileNotFound",
  }

  _NAMES_TO_VALUES = {
    "NoError": 0,
    "Server": 1,
    "Network": 2,
    "Authentication": 3,
    "Validation": 4,
    "Other": 5,
    "NotImplemented": 6,
    "FileNotFound": 7,
  }

class ArchiveStatus(object):
  Completed = 0
  InProgress = 1
  Paused = 2
  Stopped = 3
  Failed = 4
  Local = 5

  _VALUES_TO_NAMES = {
    0: "Completed",
    1: "InProgress",
    2: "Paused",
    3: "Stopped",
    4: "Failed",
    5: "Local",
  }

  _NAMES_TO_VALUES = {
    "Completed": 0,
    "InProgress": 1,
    "Paused": 2,
    "Stopped": 3,
    "Failed": 4,
    "Local": 5,
  }

class CertExportFormat(object):
  HTML = 0
  YAML = 1
  PDF = 2

  _VALUES_TO_NAMES = {
    0: "HTML",
    1: "YAML",
    2: "PDF",
  }

  _NAMES_TO_VALUES = {
    "HTML": 0,
    "YAML": 1,
    "PDF": 2,
  }


class InvalidOperation(TException):
  """
  Attributes:
   - what
   - why
   - filename
  """

  thrift_spec = (
    None, # 0
    (1, TType.I32, 'what', None, None, ), # 1
    (2, TType.STRING, 'why', None, None, ), # 2
    (3, TType.STRING, 'filename', None, None, ), # 3
  )

  def __init__(self, what=None, why=None, filename=None,):
    self.what = what
    self.why = why
    self.filename = filename

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.what = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.why = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.filename = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('InvalidOperation')
    if self.what is not None:
      oprot.writeFieldBegin('what', TType.I32, 1)
      oprot.writeI32(self.what)
      oprot.writeFieldEnd()
    if self.why is not None:
      oprot.writeFieldBegin('why', TType.STRING, 2)
      oprot.writeString(self.why)
      oprot.writeFieldEnd()
    if self.filename is not None:
      oprot.writeFieldBegin('filename', TType.STRING, 3)
      oprot.writeString(self.filename)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __str__(self):
    return repr(self)

  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class DateInfo(object):
  """
  Attributes:
   - Day
   - Month
   - Year
   - Hour
   - Minutes
   - Seconds
  """

  thrift_spec = (
    None, # 0
    (1, TType.I32, 'Day', None, None, ), # 1
    (2, TType.I32, 'Month', None, None, ), # 2
    (3, TType.I32, 'Year', None, None, ), # 3
    (4, TType.I32, 'Hour', None, None, ), # 4
    (5, TType.I32, 'Minutes', None, None, ), # 5
    (6, TType.I32, 'Seconds', None, None, ), # 6
  )

  def __init__(self, Day=None, Month=None, Year=None, Hour=None, Minutes=None, Seconds=None,):
    self.Day = Day
    self.Month = Month
    self.Year = Year
    self.Hour = Hour
    self.Minutes = Minutes
    self.Seconds = Seconds

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.I32:
          self.Day = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I32:
          self.Month = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I32:
          self.Year = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.I32:
          self.Hour = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.I32:
          self.Minutes = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.I32:
          self.Seconds = iprot.readI32();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('DateInfo')
    if self.Day is not None:
      oprot.writeFieldBegin('Day', TType.I32, 1)
      oprot.writeI32(self.Day)
      oprot.writeFieldEnd()
    if self.Month is not None:
      oprot.writeFieldBegin('Month', TType.I32, 2)
      oprot.writeI32(self.Month)
      oprot.writeFieldEnd()
    if self.Year is not None:
      oprot.writeFieldBegin('Year', TType.I32, 3)
      oprot.writeI32(self.Year)
      oprot.writeFieldEnd()
    if self.Hour is not None:
      oprot.writeFieldBegin('Hour', TType.I32, 4)
      oprot.writeI32(self.Hour)
      oprot.writeFieldEnd()
    if self.Minutes is not None:
      oprot.writeFieldBegin('Minutes', TType.I32, 5)
      oprot.writeI32(self.Minutes)
      oprot.writeFieldEnd()
    if self.Seconds is not None:
      oprot.writeFieldBegin('Seconds', TType.I32, 6)
      oprot.writeI32(self.Seconds)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class TransferStatus(object):
  """
  Attributes:
   - StatusDescription
   - ETA
   - RemainingBytes
   - Progress
   - Status
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'StatusDescription', None, None, ), # 1
    (2, TType.STRING, 'ETA', None, None, ), # 2
    (3, TType.I64, 'RemainingBytes', None, None, ), # 3
    (4, TType.DOUBLE, 'Progress', None, None, ), # 4
    (5, TType.I32, 'Status', None, None, ), # 5
  )

  def __init__(self, StatusDescription=None, ETA=None, RemainingBytes=None, Progress=None, Status=None,):
    self.StatusDescription = StatusDescription
    self.ETA = ETA
    self.RemainingBytes = RemainingBytes
    self.Progress = Progress
    self.Status = Status

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.StatusDescription = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.ETA = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I64:
          self.RemainingBytes = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.DOUBLE:
          self.Progress = iprot.readDouble();
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.I32:
          self.Status = iprot.readI32();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('TransferStatus')
    if self.StatusDescription is not None:
      oprot.writeFieldBegin('StatusDescription', TType.STRING, 1)
      oprot.writeString(self.StatusDescription)
      oprot.writeFieldEnd()
    if self.ETA is not None:
      oprot.writeFieldBegin('ETA', TType.STRING, 2)
      oprot.writeString(self.ETA)
      oprot.writeFieldEnd()
    if self.RemainingBytes is not None:
      oprot.writeFieldBegin('RemainingBytes', TType.I64, 3)
      oprot.writeI64(self.RemainingBytes)
      oprot.writeFieldEnd()
    if self.Progress is not None:
      oprot.writeFieldBegin('Progress', TType.DOUBLE, 4)
      oprot.writeDouble(self.Progress)
      oprot.writeFieldEnd()
    if self.Status is not None:
      oprot.writeFieldBegin('Status', TType.I32, 5)
      oprot.writeI32(self.Status)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class ArchiveInfo(object):
  """
  Attributes:
   - Title
   - Description
   - SizeInBytes
   - CreatedDate
   - Md5HexDigits
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'Title', None, None, ), # 1
    (2, TType.STRING, 'Description', None, None, ), # 2
    (3, TType.I64, 'SizeInBytes', None, None, ), # 3
    (4, TType.STRUCT, 'CreatedDate', (DateInfo, DateInfo.thrift_spec), None, ), # 4
    (5, TType.STRING, 'Md5HexDigits', None, None, ), # 5
  )

  def __init__(self, Title=None, Description=None, SizeInBytes=None, CreatedDate=None, Md5HexDigits=None,):
    self.Title = Title
    self.Description = Description
    self.SizeInBytes = SizeInBytes
    self.CreatedDate = CreatedDate
    self.Md5HexDigits = Md5HexDigits

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.Title = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.Description = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.I64:
          self.SizeInBytes = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRUCT:
          self.CreatedDate = DateInfo()
          self.CreatedDate.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.STRING:
          self.Md5HexDigits = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('ArchiveInfo')
    if self.Title is not None:
      oprot.writeFieldBegin('Title', TType.STRING, 1)
      oprot.writeString(self.Title)
      oprot.writeFieldEnd()
    if self.Description is not None:
      oprot.writeFieldBegin('Description', TType.STRING, 2)
      oprot.writeString(self.Description)
      oprot.writeFieldEnd()
    if self.SizeInBytes is not None:
      oprot.writeFieldBegin('SizeInBytes', TType.I64, 3)
      oprot.writeI64(self.SizeInBytes)
      oprot.writeFieldEnd()
    if self.CreatedDate is not None:
      oprot.writeFieldBegin('CreatedDate', TType.STRUCT, 4)
      self.CreatedDate.write(oprot)
      oprot.writeFieldEnd()
    if self.Md5HexDigits is not None:
      oprot.writeFieldBegin('Md5HexDigits', TType.STRING, 5)
      oprot.writeString(self.Md5HexDigits)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Capsule(object):
  """
  Attributes:
   - Created
   - ID
   - Resource_URI
   - Title
   - User
   - ExpirationDate
   - TotalSizeInBytes
   - AvailableSizeInBytes
   - CapsuleContents
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'Created', None, None, ), # 1
    (2, TType.STRING, 'ID', None, None, ), # 2
    (3, TType.STRING, 'Resource_URI', None, None, ), # 3
    (4, TType.STRING, 'Title', None, None, ), # 4
    (5, TType.STRING, 'User', None, None, ), # 5
    (6, TType.STRUCT, 'ExpirationDate', (DateInfo, DateInfo.thrift_spec), None, ), # 6
    (7, TType.I64, 'TotalSizeInBytes', None, None, ), # 7
    (8, TType.I64, 'AvailableSizeInBytes', None, None, ), # 8
    (9, TType.LIST, 'CapsuleContents', (TType.STRUCT,(ArchiveInfo, ArchiveInfo.thrift_spec)), None, ), # 9
  )

  def __init__(self, Created=None, ID=None, Resource_URI=None, Title=None, User=None, ExpirationDate=None, TotalSizeInBytes=None, AvailableSizeInBytes=None, CapsuleContents=None,):
    self.Created = Created
    self.ID = ID
    self.Resource_URI = Resource_URI
    self.Title = Title
    self.User = User
    self.ExpirationDate = ExpirationDate
    self.TotalSizeInBytes = TotalSizeInBytes
    self.AvailableSizeInBytes = AvailableSizeInBytes
    self.CapsuleContents = CapsuleContents

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.Created = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.ID = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.Resource_URI = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.Title = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.STRING:
          self.User = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 6:
        if ftype == TType.STRUCT:
          self.ExpirationDate = DateInfo()
          self.ExpirationDate.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 7:
        if ftype == TType.I64:
          self.TotalSizeInBytes = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 8:
        if ftype == TType.I64:
          self.AvailableSizeInBytes = iprot.readI64();
        else:
          iprot.skip(ftype)
      elif fid == 9:
        if ftype == TType.LIST:
          self.CapsuleContents = []
          (_etype3, _size0) = iprot.readListBegin()
          for _i4 in xrange(_size0):
            _elem5 = ArchiveInfo()
            _elem5.read(iprot)
            self.CapsuleContents.append(_elem5)
          iprot.readListEnd()
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Capsule')
    if self.Created is not None:
      oprot.writeFieldBegin('Created', TType.STRING, 1)
      oprot.writeString(self.Created)
      oprot.writeFieldEnd()
    if self.ID is not None:
      oprot.writeFieldBegin('ID', TType.STRING, 2)
      oprot.writeString(self.ID)
      oprot.writeFieldEnd()
    if self.Resource_URI is not None:
      oprot.writeFieldBegin('Resource_URI', TType.STRING, 3)
      oprot.writeString(self.Resource_URI)
      oprot.writeFieldEnd()
    if self.Title is not None:
      oprot.writeFieldBegin('Title', TType.STRING, 4)
      oprot.writeString(self.Title)
      oprot.writeFieldEnd()
    if self.User is not None:
      oprot.writeFieldBegin('User', TType.STRING, 5)
      oprot.writeString(self.User)
      oprot.writeFieldEnd()
    if self.ExpirationDate is not None:
      oprot.writeFieldBegin('ExpirationDate', TType.STRUCT, 6)
      self.ExpirationDate.write(oprot)
      oprot.writeFieldEnd()
    if self.TotalSizeInBytes is not None:
      oprot.writeFieldBegin('TotalSizeInBytes', TType.I64, 7)
      oprot.writeI64(self.TotalSizeInBytes)
      oprot.writeFieldEnd()
    if self.AvailableSizeInBytes is not None:
      oprot.writeFieldBegin('AvailableSizeInBytes', TType.I64, 8)
      oprot.writeI64(self.AvailableSizeInBytes)
      oprot.writeFieldEnd()
    if self.CapsuleContents is not None:
      oprot.writeFieldBegin('CapsuleContents', TType.LIST, 9)
      oprot.writeListBegin(TType.STRUCT, len(self.CapsuleContents))
      for iter6 in self.CapsuleContents:
        iter6.write(oprot)
      oprot.writeListEnd()
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Archive(object):
  """
  Attributes:
   - LocalID
   - Status
   - Info
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'LocalID', None, None, ), # 1
    (2, TType.I32, 'Status', None, None, ), # 2
    (3, TType.STRUCT, 'Info', (ArchiveInfo, ArchiveInfo.thrift_spec), None, ), # 3
  )

  def __init__(self, LocalID=None, Status=None, Info=None,):
    self.LocalID = LocalID
    self.Status = Status
    self.Info = Info

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.LocalID = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.I32:
          self.Status = iprot.readI32();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRUCT:
          self.Info = ArchiveInfo()
          self.Info.read(iprot)
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Archive')
    if self.LocalID is not None:
      oprot.writeFieldBegin('LocalID', TType.STRING, 1)
      oprot.writeString(self.LocalID)
      oprot.writeFieldEnd()
    if self.Status is not None:
      oprot.writeFieldBegin('Status', TType.I32, 2)
      oprot.writeI32(self.Status)
      oprot.writeFieldEnd()
    if self.Info is not None:
      oprot.writeFieldBegin('Info', TType.STRUCT, 3)
      self.Info.write(oprot)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Signature(object):
  """
  Attributes:
   - ArchiveID
   - DateCreated
   - UploaderName
   - UploaderEmail
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'ArchiveID', None, None, ), # 1
    (2, TType.STRUCT, 'DateCreated', (DateInfo, DateInfo.thrift_spec), None, ), # 2
    (3, TType.STRING, 'UploaderName', None, None, ), # 3
    (4, TType.STRING, 'UploaderEmail', None, None, ), # 4
  )

  def __init__(self, ArchiveID=None, DateCreated=None, UploaderName=None, UploaderEmail=None,):
    self.ArchiveID = ArchiveID
    self.DateCreated = DateCreated
    self.UploaderName = UploaderName
    self.UploaderEmail = UploaderEmail

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.ArchiveID = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.DateCreated = DateInfo()
          self.DateCreated.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.UploaderName = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.UploaderEmail = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Signature')
    if self.ArchiveID is not None:
      oprot.writeFieldBegin('ArchiveID', TType.STRING, 1)
      oprot.writeString(self.ArchiveID)
      oprot.writeFieldEnd()
    if self.DateCreated is not None:
      oprot.writeFieldBegin('DateCreated', TType.STRUCT, 2)
      self.DateCreated.write(oprot)
      oprot.writeFieldEnd()
    if self.UploaderName is not None:
      oprot.writeFieldBegin('UploaderName', TType.STRING, 3)
      oprot.writeString(self.UploaderName)
      oprot.writeFieldEnd()
    if self.UploaderEmail is not None:
      oprot.writeFieldBegin('UploaderEmail', TType.STRING, 4)
      oprot.writeString(self.UploaderEmail)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Certificate(object):
  """
  Attributes:
   - HexDigitsKey
   - Sig
   - RelatedArchive
   - LocalID
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'HexDigitsKey', None, None, ), # 1
    (2, TType.STRUCT, 'Sig', (Signature, Signature.thrift_spec), None, ), # 2
    (3, TType.STRUCT, 'RelatedArchive', (ArchiveInfo, ArchiveInfo.thrift_spec), None, ), # 3
    (4, TType.STRING, 'LocalID', None, None, ), # 4
  )

  def __init__(self, HexDigitsKey=None, Sig=None, RelatedArchive=None, LocalID=None,):
    self.HexDigitsKey = HexDigitsKey
    self.Sig = Sig
    self.RelatedArchive = RelatedArchive
    self.LocalID = LocalID

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.HexDigitsKey = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRUCT:
          self.Sig = Signature()
          self.Sig.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRUCT:
          self.RelatedArchive = ArchiveInfo()
          self.RelatedArchive.read(iprot)
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.LocalID = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Certificate')
    if self.HexDigitsKey is not None:
      oprot.writeFieldBegin('HexDigitsKey', TType.STRING, 1)
      oprot.writeString(self.HexDigitsKey)
      oprot.writeFieldEnd()
    if self.Sig is not None:
      oprot.writeFieldBegin('Sig', TType.STRUCT, 2)
      self.Sig.write(oprot)
      oprot.writeFieldEnd()
    if self.RelatedArchive is not None:
      oprot.writeFieldBegin('RelatedArchive', TType.STRUCT, 3)
      self.RelatedArchive.write(oprot)
      oprot.writeFieldEnd()
    if self.LocalID is not None:
      oprot.writeFieldBegin('LocalID', TType.STRING, 4)
      oprot.writeString(self.LocalID)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class Settings(object):
  """
  Attributes:
   - StoredUserName
   - StoredPassword
   - RememberMe
   - ArchivesFolder
   - CertificatesFolder
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'StoredUserName', None, None, ), # 1
    (2, TType.STRING, 'StoredPassword', None, None, ), # 2
    (3, TType.BOOL, 'RememberMe', None, None, ), # 3
    (4, TType.STRING, 'ArchivesFolder', None, None, ), # 4
    (5, TType.STRING, 'CertificatesFolder', None, None, ), # 5
  )

  def __init__(self, StoredUserName=None, StoredPassword=None, RememberMe=None, ArchivesFolder=None, CertificatesFolder=None,):
    self.StoredUserName = StoredUserName
    self.StoredPassword = StoredPassword
    self.RememberMe = RememberMe
    self.ArchivesFolder = ArchivesFolder
    self.CertificatesFolder = CertificatesFolder

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.StoredUserName = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.StoredPassword = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.BOOL:
          self.RememberMe = iprot.readBool();
        else:
          iprot.skip(ftype)
      elif fid == 4:
        if ftype == TType.STRING:
          self.ArchivesFolder = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 5:
        if ftype == TType.STRING:
          self.CertificatesFolder = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('Settings')
    if self.StoredUserName is not None:
      oprot.writeFieldBegin('StoredUserName', TType.STRING, 1)
      oprot.writeString(self.StoredUserName)
      oprot.writeFieldEnd()
    if self.StoredPassword is not None:
      oprot.writeFieldBegin('StoredPassword', TType.STRING, 2)
      oprot.writeString(self.StoredPassword)
      oprot.writeFieldEnd()
    if self.RememberMe is not None:
      oprot.writeFieldBegin('RememberMe', TType.BOOL, 3)
      oprot.writeBool(self.RememberMe)
      oprot.writeFieldEnd()
    if self.ArchivesFolder is not None:
      oprot.writeFieldBegin('ArchivesFolder', TType.STRING, 4)
      oprot.writeString(self.ArchivesFolder)
      oprot.writeFieldEnd()
    if self.CertificatesFolder is not None:
      oprot.writeFieldBegin('CertificatesFolder', TType.STRING, 5)
      oprot.writeString(self.CertificatesFolder)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)

class VersionInfo(object):
  """
  Attributes:
   - version
   - description
   - uri
  """

  thrift_spec = (
    None, # 0
    (1, TType.STRING, 'version', None, None, ), # 1
    (2, TType.STRING, 'description', None, None, ), # 2
    (3, TType.STRING, 'uri', None, None, ), # 3
  )

  def __init__(self, version=None, description=None, uri=None,):
    self.version = version
    self.description = description
    self.uri = uri

  def read(self, iprot):
    if iprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and isinstance(iprot.trans, TTransport.CReadableTransport) and self.thrift_spec is not None and fastbinary is not None:
      fastbinary.decode_binary(self, iprot.trans, (self.__class__, self.thrift_spec))
      return
    iprot.readStructBegin()
    while True:
      (fname, ftype, fid) = iprot.readFieldBegin()
      if ftype == TType.STOP:
        break
      if fid == 1:
        if ftype == TType.STRING:
          self.version = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 2:
        if ftype == TType.STRING:
          self.description = iprot.readString();
        else:
          iprot.skip(ftype)
      elif fid == 3:
        if ftype == TType.STRING:
          self.uri = iprot.readString();
        else:
          iprot.skip(ftype)
      else:
        iprot.skip(ftype)
      iprot.readFieldEnd()
    iprot.readStructEnd()

  def write(self, oprot):
    if oprot.__class__ == TBinaryProtocol.TBinaryProtocolAccelerated and self.thrift_spec is not None and fastbinary is not None:
      oprot.trans.write(fastbinary.encode_binary(self, (self.__class__, self.thrift_spec)))
      return
    oprot.writeStructBegin('VersionInfo')
    if self.version is not None:
      oprot.writeFieldBegin('version', TType.STRING, 1)
      oprot.writeString(self.version)
      oprot.writeFieldEnd()
    if self.description is not None:
      oprot.writeFieldBegin('description', TType.STRING, 2)
      oprot.writeString(self.description)
      oprot.writeFieldEnd()
    if self.uri is not None:
      oprot.writeFieldBegin('uri', TType.STRING, 3)
      oprot.writeString(self.uri)
      oprot.writeFieldEnd()
    oprot.writeFieldStop()
    oprot.writeStructEnd()

  def validate(self):
    return


  def __repr__(self):
    L = ['%s=%r' % (key, value)
      for key, value in self.__dict__.iteritems()]
    return '%s(%s)' % (self.__class__.__name__, ', '.join(L))

  def __eq__(self, other):
    return isinstance(other, self.__class__) and self.__dict__ == other.__dict__

  def __ne__(self, other):
    return not (self == other)



from testtools import TestCase
from mock import MagicMock, patch


class MPConnectionTest(TestCase):
    def setUp(self):
        super(MPConnectionTest, self).setUp()

    def tearDown(self):
        super(MPConnectionTest, self).tearDown()

    def _makeit(self, *args, **kw):
        from lacli.pool import MPConnection
        return MPConnection(*args,  **kw)

    def _token(self, uid=None):
        return {
            'token_access_key': None,
            'token_secret_key': None,
            'token_session': None,
            'token_expiration': None,
            'token_uid': uid,
            'bucket': None
            }

    def test_makeit(self):
        assert self._makeit(MagicMock(), 4)

    def test_no_expiration(self):
        conn = self._makeit(self._token("lala"), 4)
        self.assertEqual(None, conn.timeout())

    @patch('lacli.pool.getLogger', create=True)
    def test_timeout_parse_error(self, log):
        token = self._token("lala")
        token['token_expiration'] = "foobar"
        conn = self._makeit(token, 4)
        self.assertEqual(None, conn.timeout())
        log.assert_called_with()
        log.return_value.debug.assert_called_with(
            'invalid token expiration: %s', 'foobar')
